/**
  *  \file planetscentral/talk/permission.cc
  *  \brief Talk: Permission Checks
  */

#include "planetscentral/talk/permission.h"
#include "cpluslib/string.h"
#include "planetscentral/talk/user.h"
#include "planetscentral/dbclient/dbsubtree.h"
#include "planetscentral/talk/db.h"
#include "planetscentral/talk/talk.h"

/** Privilege check.
    \param privString privilege string to check against
    \param user user to check */
bool
checkUserPermission(string_t privString, string_t user)
{
    if (privString.empty()) {
        return false;
    }

    // Process
    do {
        string_t me(strFirst(privString, ","));
        bool result = true;
        if (me.size() > 0 && me[0] == '-') {
            // Invert result
            result = false;
            me.erase(0, 1);
        }
        if (me == "all") {
            // blanket permission
            return result;
        } else if (me.size() > 2 && me.compare(0, 2, "p:", 2) == 0) {
            // check profile permission
            std::auto_ptr<IntValue> value(User(database_connection, user).getProfileRaw(me.substr(2)));
            if (toIntResult(value.get()) > 0) {
                return result;
            }
        } else if (me.size() > 2 && me.compare(0, 2, "u:", 2) == 0) {
            // check user name
            if (me.compare(2, string_t::npos, user) == 0) {
                return result;
            }
        } else if (me.size() > 2 && me.compare(0, 2, "g:", 2) == 0) {
            // check game membership
            if (+DbSubtree(database_connection, GAME_ROOT).subtree(me.substr(2)).hashKey("users").hasField(user)) {
                return result;
            }
        } else {
            // unknown privilege token; in particular
        }
    } while (strRemove(privString, ","));
    return false;
}
